#include "TChain.h"
#include "TTree.h"
#include "TH1F.h"
#include "TH2F.h"
#include "TList.h"
#include "TCanvas.h"

#include "AliAnalysisTask.h"
#include "AliAnalysisManager.h"

#include "AliCFTrackKineCuts.h"
#include "AliCFTrackQualityCuts.h"
#include "AliCFContainer.h"

#include "AliVEvent.h"
#include "AliESDEvent.h"
#include "AliMCEvent.h"
#include "AliAODEvent.h"

#include "AliAnalysisTaskPtCF.h"

// example of an analysis task creating a p_t spectrum
// Authors: Panos Cristakoglou, Jan Fiete Grosse-Oetringhaus, Christian Klein-Boesing
// extended with correction framework related operations
// Authors: Renaud Vernet, Annalisa Mastroserio, Ingrid Kraus

ClassImp(AliAnalysisTaskPtCF)

//________________________________________________________________________
AliAnalysisTaskPtCF::AliAnalysisTaskPtCF() 
  : AliAnalysisTaskSE(), fHistPtMC(0), fHistPtRec(0), fKineCutsMC(0x0), fKineCutsRec(0x0), fQualityCuts(0x0), fQAList(0x0), fAliCFContainer(0x0)
{
  // Default Constructor
}

//________________________________________________________________________
AliAnalysisTaskPtCF::AliAnalysisTaskPtCF(const char *name) 
  : AliAnalysisTaskSE(name), fHistPtMC(0), fHistPtRec(0), fKineCutsMC(0x0), fKineCutsRec(0x0), fQualityCuts(0x0), fQAList(0x0), fAliCFContainer(0x0)
{
  // Constructor
  // Define input and output slots here
  // Input slot #0 works with a TChain
  // Output slot #1 writes into a TH1 container
  // Output slot #2 writes into a TH1 container
  // Output slot #3 writes into a correction framework type container
  // Output slot #4 writes into a TList container
  DefineOutput(1, TH1F::Class());
  DefineOutput(2, TH1F::Class());
  DefineOutput(3, AliCFContainer::Class());
  DefineOutput(4, TList::Class());
}

//________________________________________________________________________
AliAnalysisTaskPtCF& AliAnalysisTaskPtCF::operator=(const AliAnalysisTaskPtCF& c)
{
  //
  // Assignment operator
  //
  if (this!=&c) {
    AliAnalysisTaskSE::operator=(c);
    fHistPtMC = c.fHistPtMC;
    fHistPtRec = c.fHistPtRec;
    fKineCutsMC = c.fKineCutsMC;
    fKineCutsRec = c.fKineCutsRec;
    fQualityCuts = c.fQualityCuts;
    fQAList = c.fQAList;
    fAliCFContainer = c.fAliCFContainer;
  }
  return *this;
}

//________________________________________________________________________
AliAnalysisTaskPtCF::AliAnalysisTaskPtCF(const AliAnalysisTaskPtCF& c) :
  AliAnalysisTaskSE(c),
  fHistPtMC(c.fHistPtMC),
  fHistPtRec(c.fHistPtRec),
  fKineCutsMC(c.fKineCutsMC),
  fKineCutsRec(c.fKineCutsRec),
  fQualityCuts(c.fQualityCuts),
  fQAList(c.fQAList),
  fAliCFContainer(c.fAliCFContainer)
{
  //
  // Copy Constructor
  //
}

//________________________________________________________________________
AliAnalysisTaskPtCF::~AliAnalysisTaskPtCF() {
  //
  //destructor
  //
  Info("~AliAnalysisTaskPtCF","Calling Destructor");
  if (fHistPtMC)	delete fHistPtMC;
  if (fHistPtRec)	delete fHistPtRec;
  if (fKineCutsMC)	delete fKineCutsMC;
  if (fKineCutsRec)	delete fKineCutsRec;
  if (fQualityCuts)	delete fQualityCuts;
  if (fQAList)		{fQAList->Clear(); delete fQAList;}
  if (fAliCFContainer)	delete fAliCFContainer;
}
//________________________________________________________________________
void AliAnalysisTaskPtCF::UserCreateOutputObjects()
{
  // Create histograms
  // Called once

  fHistPtMC = new TH1F("fHistPtMC", "MC P_{T} distribution", 15, 0.1, 3.1);
  fHistPtMC->GetXaxis()->SetTitle("P_{T} (GeV/c)");
  fHistPtMC->GetYaxis()->SetTitle("dN/dP_{T} (c/GeV)");
  fHistPtMC->SetMarkerStyle(kFullCircle);

  fHistPtRec = new TH1F("fHistPtRec", "Rec P_{T} distribution", 15, 0.1, 3.1);
  fHistPtRec->GetXaxis()->SetTitle("P_{T} (GeV/c)");
  fHistPtRec->GetYaxis()->SetTitle("dN/dP_{T} (c/GeV)");
  fHistPtRec->SetMarkerStyle(kFullCircle);
}

//________________________________________________________________________
void AliAnalysisTaskPtCF::UserExec(Option_t *) 
{
  // Main loop - generated particles
  // Called for each event

  // also a AliEvent...
  //  AliVEvent* mcEvent = MCEvent();
  AliMCEvent* mcEvent = MCEvent();
  if (!mcEvent) {
     Printf("ERROR: Could not retrieve MC event");
     return;
  }

  Printf("MC particles: %d", mcEvent->GetNumberOfTracks());

  // Track loop to fill a pT spectrum
  for (Int_t iTrack = 0; iTrack < mcEvent->GetNumberOfTracks(); iTrack++) {
    AliVParticle *track = mcEvent->GetTrack(iTrack);
    if (!track) {
      Printf("ERROR: Could not receive track %d", iTrack);
      continue;
    }
    fHistPtMC->Fill(track->Pt());

    if(!fKineCutsMC) {
      Printf("ERROR: Could not access kine cut %d", iTrack);
      continue;
    }
    // fill QA histograms before and after the cut is applied
    fKineCutsMC->FillHistograms(track,0);
    if(!fKineCutsMC->IsSelected(track)) continue;
    fKineCutsMC->FillHistograms(track,1);

    // fill container, first step is MC info: Fill(...,0)
    Double_t containerInput[2] ;
    containerInput[0] = track->Pt();
    containerInput[1] = track->Eta() ;
    fAliCFContainer->Fill(containerInput,0);
  } //track loop 
  



  // Main loop - reconstructed tracks
  // Called for each event

  AliVEvent *event = InputEvent();
  if (!event) {
     Printf("ERROR: Could not retrieve event");
     return;
  }

  if(Entry()==0){
    AliESDEvent* esd = dynamic_cast<AliESDEvent*>(event);
    AliAODEvent* aod = dynamic_cast<AliAODEvent*>(event);
    if(esd){
      Printf("We are reading from ESD");
    }
    else if(aod){
      Printf("We are reading from AOD");
    }
  }

  Printf("There are %d tracks in this event", event->GetNumberOfTracks());

  // Track loop to fill a pT spectrum
  for (Int_t iTrack = 0; iTrack < event->GetNumberOfTracks(); iTrack++) {
    AliVParticle *track = event->GetTrack(iTrack);
    if (!track) {
      Printf("ERROR: Could not receive track %d", iTrack);
      continue;
    }
    fHistPtRec->Fill(track->Pt());

    if(!fKineCutsRec) {
      Printf("ERROR: Could not access kine cut %d", iTrack);
      continue;
    }
    if(!fQualityCuts) {
      Printf("ERROR: Could not access kine cut %d", iTrack);
      continue;
    }
    // fill QA histograms before and after the cut is applied
    fKineCutsRec->FillHistograms(track,0);
    if(!fKineCutsRec->IsSelected(track)) continue;
    fKineCutsRec->FillHistograms(track,1);
    fQualityCuts->FillHistograms(track,0);
    if(!fQualityCuts->IsSelected(track)) continue;
    fQualityCuts->FillHistograms(track,1);

    // fill container, second step is after reconstruction and cuts: Fill(...,1)
    Double_t containerInput[2] ;
    containerInput[0] = track->Pt();
    containerInput[1] = track->Eta() ;
    fAliCFContainer->Fill(containerInput,1);
  } //track loop 
  

  // Post output data.
  PostData(1, fHistPtMC);
  PostData(2, fHistPtRec);
  PostData(3, fAliCFContainer);
  PostData(4, fQAList);
}      

//________________________________________________________________________
void AliAnalysisTaskPtCF::Terminate(Option_t *) 
{
  // Draw result to the screen
  // Called once at the end of the query

  fHistPtMC = dynamic_cast<TH1F*> (GetOutputData(1));
  if (!fHistPtMC) {
    Printf("ERROR: fHistPtMC not available");
    return;
  }
  fHistPtRec = dynamic_cast<TH1F*> (GetOutputData(2));
  if (!fHistPtRec) {
    Printf("ERROR: fHistPtRec not available");
    return;
  }
  fAliCFContainer = dynamic_cast<AliCFContainer*> (GetOutputData(3));
  if (!fAliCFContainer) {
    Printf("ERROR: fAliCFContainer not available");
    return;
  }
  fQAList = dynamic_cast<TList*> (GetOutputData(4));
  if (!fQAList) {
    Printf("ERROR: fQAList not available");
    return;
  }

   
  TCanvas *c1 = new TCanvas("AliAnalysisTaskPtCF","PtCF",10,10,1020,510);
  c1->Divide(2,1);
  c1->cd(1)->SetLogy();
  fHistPtMC->DrawCopy("E");
  c1->cd(2)->SetLogy();
  fHistPtRec->DrawCopy("E");

  // plot correction framework container by projecting into histograms
  // sensitive variable 0 (pt) and 1 (eta), step 0 (MC)
  TH1D* hPtMC  = fAliCFContainer->ShowProjection(0,0);
  TH1D* hEtaMC = fAliCFContainer->ShowProjection(1,0);
  // sensitive variable 0 (pt) and 1 (eta), step 1 (reconstructed)
  TH1D* hPtRec  = fAliCFContainer->ShowProjection(0,1);
  TH1D* hEtaRec = fAliCFContainer->ShowProjection(1,1);
  // first variable (pt), second variable (eta), step (MC)
  TH2D *hMC  = fAliCFContainer->ShowProjection(0,1,0);
  // first variable (pt), second variable (eta), step (reconstructed)
  TH2D *hRec = fAliCFContainer->ShowProjection(0,1,1);

  hPtMC->SetMarkerStyle(23);
  hEtaMC->SetMarkerStyle(23);
  hPtRec->SetMarkerStyle(24);
  hEtaRec->SetMarkerStyle(24);

  TCanvas * c =new TCanvas("c","container",900,600);
  c->Divide(3,2);

  c->cd(1)->SetLogy();
  hPtMC->Draw("p");
  c->cd(2)->SetLogy();
  hPtRec->Draw("p");
  c->cd(3)->SetLogz();
  hMC->Draw("colz");
  c->cd(4);
  hEtaMC->Draw("p");
  c->cd(5);
  hEtaRec->Draw("p");
  c->cd(6)->SetLogz();
  hRec->Draw("colz");

//   kineCutsMC_transverse_momentum = (TH1*)fQAList->FindObject("kineCutsMC_transverse_momentum");
//   kineCutsMC_transverse_momentum->Draw();
}
